using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Runtime.Intrinsics.Arm;
using System.Security.Cryptography;
using FitConnect;
using FitConnect.Encryption;
using FitConnect.Interfaces;
using FitConnect.Models;
using FluentAssertions;
using Jose;
using Newtonsoft.Json;
using NUnit.Framework;

namespace E2ETests;

public class SenderTest {
    private const string desitnationId = "aa3704d6-8bd7-4d40-a8af-501851f93934";
    private string _clientId = "73a8ff88-076b-4263-9a80-8ebadac97b0d";
    private string _clientSecret = "rdlXms-4ikO47AbTmmCTdzFoE4cTSt13JmSbcY5Dhsw";
    private Sender _sender;
    private IFluentSender _fluentSender;


    /// <summary>
    ///     Setup creates json file for credentials if not found
    /// </summary>
    /// <exception cref="Exception"></exception>
    [OneTimeSetUp]
    public void OneTimeSetup() {
        // relative to the project execution directory
        const string secretFile = "../../../http-client.private.env.json";

        if (!File.Exists(secretFile)) {
            // If the secret file is not found, create it with the default values
            // The file will be pretty in C#11, when """ is introduced
            File.WriteAllText(secretFile, @"
{
    ""sender"": {
        ""id"": ""00000000-0000-0000-0000-000000000000"",
        ""secret"": ""0000000000000000000000000000000000000000000"",
        ""scope"": ""send:region:DE""
    }
}");
            throw new Exception("Please fill the secret.json file with your sender credentials");
        }

        var jsonContent = File.ReadAllText(secretFile);
        var secret = JsonConvert.DeserializeObject<dynamic>(jsonContent);
        _clientId = secret.sender.id;
        _clientSecret = secret.sender.secret;
    }

    [SetUp]
    public void Setup() {
        _sender = Sender.Create(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development));

        _fluentSender = new Client(
            FitConnectEndpoints.Create(FitConnectEndpoints.EndpointType.Development),
            _clientId, _clientSecret).GetSender();
    }

    [Test]
    public void CheckIfSecretsAreValid() {
        _clientId.Should().NotBe("00000000-0000-0000-0000-000000000000");
        _clientSecret.Should().NotBe("0000000000000000000000000000000000000000000");
    }

    [Test]
    public void WithDestination_PrepareSubmissionToServer() {
        // Arrange

        // Act
        _fluentSender.WithDestination(desitnationId);

        // Assert
        _fluentSender.PublicKey.Should().NotBeNullOrEmpty();
    }

    [Test]
    public void WithAttachments_IntroduceSubmission_ShouldGetIdFromServer() {
        // Arrange
        var dut = _fluentSender
            .WithDestination(desitnationId)
            .WithServiceType("ServiceName", "urn:de:fim:leika:leistung:99400048079000");

        var attachments = new List<Attachment>();
        for (var i = 0; i < 10; i++) {
            var content = RandomNumberGenerator.GetBytes(1024);
            var attachment = new Attachment(
                Guid.NewGuid().ToString(),
                content,
                Convert.ToBase64String(SHA256.HashData(content)),
                "RandomBytes"
            );
            attachments.Add(attachment);
        }

        // Act
        dut.WithAttachments(attachments);


        // Assert
        var submission = _fluentSender.GetType().GetProperties()
            .FirstOrDefault(p => p.Name == "Submission")
            .GetValue(_fluentSender) as Submission;
        submission.Attachments.Should().HaveCount(attachments.Count);

        submission.Id.Should().NotBeNullOrWhiteSpace();
    }
}
