using System.Net;
using System.Net.Http.Json;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;
using Microsoft.Extensions.Logging;

namespace FitConnect.RestService;

public interface IRestCallService {
    public IWebProxy? Proxy { get; set; }
}

public abstract class RestCallService {
    private readonly string _baseUrl;
    private readonly ILogger? _logger;
    public string? Token { get; set; }
    public IWebProxy? Proxy { get; set; }

    protected RestCallService(string baseUrl, ILogger? logger = null) {
        _baseUrl = baseUrl;
        _logger = logger;
    }

    protected HttpClient CreateClient() {
        var clientHandler = new HttpClientHandler() {
            Proxy = Proxy
        };
        var client = new HttpClient(handler: clientHandler);
        client.DefaultRequestHeaders.Add("Accept", "application/json");
        if (Token != null)
            client.DefaultRequestHeaders.Add("Authorization", $"Bearer {Token}");
        return client;
    }

    protected async Task<string> RestCallForString(string endpoint, HttpMethod method,
        string? body = null) {
        var client = CreateClient();

        var request = new HttpRequestMessage();
        request.Method = method;
        if (body != null)
            request.Content = new StringContent(body, Encoding.UTF8, "application/json");
        request.RequestUri = new Uri($"{_baseUrl}{endpoint}");
        var response = await client.SendAsync(request);

        if (response.IsSuccessStatusCode) {
            var content = await response.Content.ReadAsStringAsync();
            return content;
        }

        _logger?.LogError("Error calling {Method} - {Endpoint} with  and {Body} = {Status}", method,
            endpoint,
            body, response.StatusCode);
        throw new HttpRequestException("Error calling FitConnect API",
            inner: new HttpRequestException($"{method} -  {request.RequestUri.ToString()}\n" +
                                            await response.Content.ReadAsStringAsync()));
    }

    protected async Task<T?> RestCall<T>(string endpoint, HttpMethod method, string? body = null) {
        var content = await RestCallForString(endpoint, method, body);
        try {
            var result = JsonSerializer.Deserialize<T>(content);

            if (result != null)
                return result;
        }catch (JsonException e) {
            _logger?.LogError("Error deserializing {Content}", content);
            throw new HttpRequestException("Error deserializing FitConnect API response",
                inner: e);
        }

        throw new ArgumentException("Can not deserialize response to type " + typeof(T).Name);
    }
}
